
#include "sngtc_server.h"

#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if.h>
#include <arpa/inet.h>
#include <errno.h>





#define MAX_ETH_DEVS 32
system_eth_cfg_t sys_cfg[MAX_ETH_DEVS];
uint32_t sys_cfg_sz=0;

int sngtc_find_mac_by_eth(uint8_t *ethdev, system_eth_cfg_t **eth_cfg)
{
	unsigned int i;

	*eth_cfg=NULL;

	for (i=0;i<sys_cfg_sz;i++) {

		if (!sys_cfg[i].init) {
			continue;
		}

		if (strncmp(sys_cfg[i].name, (char*)ethdev, sizeof(sys_cfg[i].name)) == 0) {
			*eth_cfg=&sys_cfg[i];
			return 0;
		}
	}

	return -1;
}


int sngtc_detect_eth_dev(void)
{

  	static struct ifreq ifreqs[20];
   	struct ifconf ifconf;
	struct ifreq  ifr;
   	int  nifaces, i;
	int rval;
	int sock;
	
	sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0) {
        return -1;
    }


	memset(&ifr,0,sizeof(ifr));
	memset(&ifconf,0,sizeof(ifconf));
	ifconf.ifc_buf = (char*) (ifreqs);
	ifconf.ifc_len = sizeof(ifreqs);

	if ((rval = ioctl(sock, SIOCGIFCONF , (char*) &ifconf  )) < 0 ) {
		perror("ioctl(SIOGIFCONF)");
		return -1;
	}
	
	nifaces =  ifconf.ifc_len/sizeof(struct ifreq);

	SNGTC_INFO("====================================================================\n");
	SNGTC_INFO("\n");
	SNGTC_INFO("Detected Local Ethernet Interfaces:\n");

	for(i = 0; i < nifaces && i < MAX_ETH_DEVS; i++) {

		SNGTC_DEBUG("\t%-10s\n", ifreqs[i].ifr_name);
		memset(&ifr,0,sizeof(ifr));
		strcpy(ifr.ifr_name, ifreqs[i].ifr_name);

		if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0) {
            if (! (ifr.ifr_flags & IFF_LOOPBACK)) {

				memset(&sys_cfg[sys_cfg_sz],0,sizeof(sys_cfg[sys_cfg_sz]));

				sys_cfg[sys_cfg_sz].init=1;
				
				snprintf(sys_cfg[sys_cfg_sz].name, sizeof(sys_cfg[sys_cfg_sz].name), "%s",ifreqs[i].ifr_name);
				sys_cfg[sys_cfg_sz].flags=ifr.ifr_flags;


				if (!(ifr.ifr_flags & IFF_UP)) {

					memset(&ifr,0,sizeof(ifr));
					strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
					ifr.ifr_flags=sys_cfg[sys_cfg_sz].flags|IFF_UP|IFF_RUNNING;
					if (ioctl(sock, SIOCSIFFLAGS, &ifr) != 0) {
						SNGTC_ERROR("IF=%s SIOCSIFFLAGS: %s\n",ifreqs[i].ifr_name,strerror(errno));
						continue;
					}

					sys_cfg[sys_cfg_sz].set_if_up=1;
					SNGTC_INFO("Bringing up %s \n",ifreqs[i].ifr_name);
				}

				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
                if (ioctl(sock, SIOCGIFHWADDR, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFHWADDR: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				memcpy(sys_cfg[sys_cfg_sz].mac.mac,ifr.ifr_hwaddr.sa_data,sizeof(sys_cfg[sys_cfg_sz].mac));
				sngtc_mac_addr_to_str(sys_cfg[sys_cfg_sz].mac.mac_str,sys_cfg[sys_cfg_sz].mac.mac);
				

				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
				if (ioctl(sock, SIOCGIFADDR, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFADDR: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				sys_cfg[sys_cfg_sz].ip.ip=ntohl(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr);
				sngtc_codec_ipv4_hex_to_str(sys_cfg[sys_cfg_sz].ip.ip,sys_cfg[sys_cfg_sz].ip.ip_str);
	
				memset(&ifr,0,sizeof(ifr));
				strcpy(ifr.ifr_name, ifreqs[i].ifr_name);
				if (ioctl(sock, SIOCGIFNETMASK, &ifr) != 0) {
					SNGTC_ERROR("IF=%s SIOCGIFNETMASK: %s\n",ifreqs[i].ifr_name,strerror(errno));
					continue;
				}

				sys_cfg[sys_cfg_sz].mask.ip=ntohl(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr);
				sngtc_codec_ipv4_hex_to_str(sys_cfg[sys_cfg_sz].mask.ip,sys_cfg[sys_cfg_sz].mask.ip_str);

				SNGTC_INFO("%s: MAC=%s IP=%s/%s\n",
					sys_cfg[sys_cfg_sz].name, sys_cfg[sys_cfg_sz].mac.mac_str,sys_cfg[sys_cfg_sz].ip.ip_str,sys_cfg[sys_cfg_sz].mask.ip_str);

				sys_cfg_sz++;
            }
        }
		
	}

	if (sys_cfg_sz == 0) {
		SNGTC_ERROR("\tNo Local Ethernet Intefaces Found!\n");
	}
	SNGTC_INFO("\n");
	SNGTC_INFO("====================================================================\n");

	return sys_cfg_sz;
}

